import dwf
from pyocd.gdbserver import GDBServer

import interrupts
import loader
from analog_devices import AnalogDevices
from digital_pin import DigilentDigitalOutPin
import pin_defs
from lcd import LCDSpyDevice
from leds import Leds
from buttons import Buttons
import time
from spi_device import SPISpyDevice
import serial
import serial.tools.list_ports
from elftools.elf.elffile import ELFFile
from voltage_manager import VoltageManager
from servo import Servo


class NoMCUConnectedError:
    pass


def setup_serial_contains(device_string: str):
    ports = serial.tools.list_ports.comports()
    for port, desc, _ in ports:
        if device_string in desc:
            return serial.Serial(port, 115200, timeout=10)
    raise NoMCUConnectedError


class EmTest:

    def setup_class(self):
        try:
            week_branch = self.week_branch
        except AttributeError:
            assert False
        try:
            replacements = self.replacements
        except AttributeError:
            replacements = []
        try:
            generated_replacements = self.generated_replacements
        except AttributeError:
            generated_replacements = []
        try:
            to_delete = self.to_delete
        except AttributeError:
            to_delete = []
        self.session, self.file = loader.load_and_open_session(week_branch, replacements, generated_replacements, to_delete)
        self.digilent_device = dwf.Dwf()
        self.leds = Leds(self.digilent_device)
        self.buttons = Buttons(self.digilent_device)
        self.analog = AnalogDevices(self.digilent_device)
        self.spi = SPISpyDevice(self.digilent_device)
        self.serial = setup_serial_contains('DAPLink')
        self.uart = setup_serial_contains('UART')
        self.multiplexer = DigilentDigitalOutPin(self.digilent_device, pin_defs.MULTIPLEXER_PIN_POS)
        self.lcd = LCDSpyDevice(self.digilent_device)
        self.elf_file = ELFFile(self.file)
        self.interrupts = interrupts.Interrupts(self.session.target, self.elf_file)
        self.voltage_manager = VoltageManager(self.digilent_device)
        self.voltage_manager.enable_5v()
        self.servo = Servo(self.digilent_device, angle_adjustment=45)

    def teardown_class(self):
        self.session.target.reset_and_halt()
        self.session.close()
        self.digilent_device.close()
        self.serial.close()
        self.uart.close()
        self.file.close()

    def reset_peripherals(self):
        self.buttons.reset()
        self.serial.reset_input_buffer()
        self.multiplexer.reset()
        self.lcd.reset()

    def setup_method(self, method):
        self.session.target.reset_and_halt()
        self.reset_peripherals()
        self.session.target.resume()
        time.sleep(0.1)
